function [beta,result]=SingleProduct(s,df,dc,N,g)
% SINGLEPRODUCT   Computes the equilibrium in parallel strategies for the
% case of single product firms (N=J).
%
% SingleProduct(g,n,dc,df,s) computes the eq. using the following baseline
% dynamic parameters:
% g: size of the young cohort where old=1. Parameter g in the paper
% n: number of firms/products N=J. Parameter N (or J) in the paper.
% dc: Consumers' discount rate. \delta_{c} in the paper.
% df: Firms' "effective" discount rate. \delta_{f} in the paper.
% s: switching costs. s=s/L in the paper, where L is the lenght of each
% segment which is normalized to 1.
%
% Output: [beta,result]
% - beta is the slope of the policy function in the parallel strategies.
% - result is a structure containing several characteristics of the 
% equilibrium:
% result.c: quadratic term in the value function (C, in the paper).
% result.xi0: Intercept in Eq 70.
% result.xi1: Slope of competitors' average alpha in Eq 70.
% result.mu0: Intercept in the policy function in Eq 30.
% result.mu1: Slope of (c_{i}-\overline{c}) in the policy function in Eq 30.
% result.pss0: Average markup (or price if we normalized costs=0) in  Eq 31.
% result.kappad: long-run sensitivity of young consumers demand (\kappa_{X}
% divided by (J-1)).
% result.kappam=long-run sensitivity of young consumers demand (\kappa_{M}
% divided by (J-1)).
% Equation (19) consumers' average costs-  gCost(young)+Cost(old).
% result.gammaeq: Equilibrium passthrough (\gamma^{e} in Eq 34).
% result.CACvarP: coefficient of the variance of prices in the 
% result.CACcons: constant term in the consumers' average costs.
% Other variables not used in the paper: 
% result.CECvarP and result.CECcons: coefs for an alternative welfare measure
% defined as the consumer's expected cost: Cost(young)+dc*Cost(old);
% result.CC3varP and result.CC3cons: coefs for an alternative welfare measure
% defined as: g*Cost(young)+dc*Cost(old);

k=ones(max([numel(s) numel(df) numel(dc) numel(N) numel(g)]),1);
s=s(:).*k;
df=df(:).*k;
dc=dc(:).*k;
N=N(:).*k;
g=g(:).*k;
polynomial=[dc.*N.*s.*(df+dc.^2.*(1+(-2).*N).*s.^2),...\quartic coef
   N.*(df.*(1+g+dc.*((-2)+N).*s.^2)+dc.^2.*s.^2.*(3+g+(-2).*g.*N+N.*(...
   (-6)+dc.*s.^2))),...\cubic coef
   N.*s.*(df.*((-2)+N)+(-1).*dc.*(3+2.*g).*((-1)+2.*N)+3.*dc.^2.*N.* ...
   s.^2),... \quadratic coef
   (-1).*N.*((1+g).*((-1)+2.*N)+(-3).*dc.*N.*s.^2),... \linear coef
   N.^2.*s]; %\intercept
upperlimit=(1+(-1).*N).*((-1)+dc.*s).^(-1);
upperlimit(isinf(upperlimit))=Inf;
rootsp=zeros(length(s),4);
for i=1:length(s)
    r=roots(polynomial(i,:));
    rootsp(i,:)=[-ones(4-length(r),1); r];
end
rootsp=rootsp.*repmat((N-1),1,4); %Here I multiply the roots by (N-1)
rootsp(imag(rootsp)~=0)=-1;
rootsp=real(rootsp);
rootsp(rootsp>repmat(upperlimit,1,4))=-1;
beta=max(rootsp,[],2);
beta(beta<0)=NaN;

c=beta.*((-1)+N).^(-1).*((-1)+N+beta.*dc.*s).*((-1).*beta.^2.*dc.*s+ ...
  ((-1)+N).^2.*s+(-1).*beta.*((-1)+N).*(1+g+(-1).*dc.*s.^2)).*(((-1) ...
  +N).^2+2.*beta.*dc.*((-1)+N).*s+beta.^2.*((-1).*df+dc.^2.*s.^2)) ...
  .^(-1);
result.c=c;
xi0=((-1)+N).^(-1).*((-2).*c.*df.*((-1)+N).^2.*((-1)+beta+N+beta.*dc.* ...
  s)+(beta.*df+N.*((-1)+N+beta.*dc.*s)).*(g.*((-1)+N).*((-1)+beta+N+ ...
  beta.*dc.*s)+((-1)+N+beta.*dc.*s).*((-1)+beta+N+s+(-1).*N.*s))).*( ...
  beta.^2.*dc.*s.*(df+2.*dc.*N.*s)+beta.*((-1)+N).*(df.*(1+g+dc.*g.* ...
  s+dc.*N.*s.^2)+dc.*N.*s.*(4+g.*(2+dc.*s)))+((-1)+N).^2.*((-2).*c.* ...
  df.*(1+dc.*s)+N.*(2+df.*s+g.*(2+dc.*s)))).^(-1);
result.xi0=xi0;
xi1=((-2).*c.*df.*((-1)+N).^2.*(1+dc.*s)+(beta.*df+N.*((-1)+N+beta.* ...
  dc.*s)).*(beta.*dc.*s+((-1)+N).*(1+g+dc.*g.*s))).*(beta.^2.*dc.* ...
  s.*(df+2.*dc.*N.*s)+beta.*((-1)+N).*(df.*(1+g+dc.*g.*s+dc.*N.* ...
  s.^2)+dc.*N.*s.*(4+g.*(2+dc.*s)))+((-1)+N).^2.*((-2).*c.*df.*(1+ ...
  dc.*s)+N.*(2+df.*s+g.*(2+dc.*s)))).^(-1);
result.xi1=xi1;
result.mu0=xi0./(1-xi1);
result.mu1=(-1).*((-1)+N).*((-1)+xi1).*((-1)+N+xi1).^(-1);
kappad=((-1)+N).^(-1).*(1+dc.*s);
kappam=(1+g).^(-1).*((-1)+N).^(-1).*(1+g+s+dc.*g.*s+dc.*s.^2);
%kappap=1+(-1).*N.*xi1.*((-1)+N+xi1).^(-1); Deprecated. Same as result.mu1
%result.kappap=kappap;
gammaeq=result.mu1./(1+beta.*kappad);
result.pss0=result.mu0+beta./N;
result.kappad=kappad;
result.kappam=kappam;
result.gammaeq=gammaeq;
% Welfare: CAC consumer average cost: weights gCost(young)+Cost(old). See
% WelfareAnalysis.nb (This is the measure of welfare used in the
% paper).
result.CACvarP=(-1/2).*(1+g).^(-1).*(1+g+2.*s+dc.*(2+(-1).*dc.*g).*s.^2);
result.CACcons=(4+4.*g).^(-1).*(1+g+2.*s+(-1).*s.^2);
% Welfare: CEC consumer expected cost: weights Cost(young)+dc*Cost(old).
% See WelfareAnalysis.nb 
result.CECvarP=(-1/2).*(1+dc).^(-1).*(1+dc.*(1+s.*(2+dc.*s)));
result.CECcons=(4+4.*dc).^(-1).*(1+dc+2.*dc.*s+(-1).*dc.*s.^2);
% Welfare: Third alternative: weights gCost(young)+dc*Cost(old). See
% WelfareAnalysis.nb
result.CC3varP=(-1/2).*(dc+g).^(-1).*(g+dc.*(1+s.*(2+(-1).*dc.*((-2)+g).*s)));
result.CC3cons=(1/4).*(dc+g).^(-1).*(g+dc.*(1+(-1).*((-2)+s).*s));


