function [Pti dist V]=bestRespAsymg(P,fmat,c,s,dc,df,g,i)
%bestResmpAsymg  Compute the best response of a firm to competitors'
%strategies in the General Asymmetric case. 
% bestRespAsymg(P,fmat,c,L,s,dc,df,g,i), where:
% P: N-by-N matrix (N is the number of firms). Competitors and firms'
% future incarnations strategies such that the vector of prices is p=Px,
% where x is the vector of market shares.
% fmat: N-by-N symmetric matrix. The element ij indicates the density of
% consumers in segments between firms i and j. The element ii indicates the
% density in segments trapped by firm i. This matrix has to be symmetric
% and its elements in the lower (upper) triangular matrix have to add to 1.
% c: Nx1 vector. Vector of costs.
% s: scalar. Switching costs
% dc: scalar. Consumers' discount rate
% df: scalar. Firms' effective discount rate (r_{f} times g).
% g: scalar. Size of the young cohort relative to the old cohort.
% i: integer. Firm whose best response we are calculating.
% Output:
% Pti: Nx1 vector. Best response of firm i.
% dist: scalar. Measure of convergence of the value function iteration
% V: NxN symmetric matrix. The value function is V(x)=x'Vx.
% Created by Paulo Somaini on December 7th, 2009
% Dependent files: None
% Note: L, the length of each segment is set to 1.
L=1;
% Number of active firms;
N=size(fmat,1); 
% Check fmat;
isSym=max(max(abs(fmat-fmat')));
sumOne=abs(sum(sum(tril(fmat)))-1);
if isSym+sumOne>1e-9; error('fmat has to be symmetric and its lower (upper) triangular matrix has to add to one'); end
f=fmat-tril(triu(fmat));
free=sum(sum(f))/2;
% Coefficients of the demand for old consumers (Eq 78);
Oi=f(i,:)*P/(2*L); % Begins to construct d_{k,x}^{old}.
Oi(i)=Oi(i)+s*free; % Adds the term s*f*x_{k}
Oi1=(1-s)/2*sum(f(i,:)); %the constant term.
Oi=Oi+Oi1; %Add the constant term. Ends term d_{k,x}^{old}.
Oip=-sum(f(i,:))/(2*L); % d_{k,p}^{old}.
% Young Consumers (Eq 80);
dyi1=1/2*sum(f(i,:)); %the constant term
dyip=-sum(f(i,:))/(2*L); %own price coefficient
zi=f(i,:)*P/(2*L); %competitors' prices coefficient
wi=-dc*s/(2*L)*(sum(f(i,:))*P(i,:)-f(i,:)*P); %coefficients of future prices.
% Competitors' demand from young consumers (Eq 81).
dyj1=1/2*sum(f,2); %the constant term
dyjp=f(:,i)/(2*L); %coefficient of price.
Z=zeros(N,N);
W=zeros(N,N);
fj_i=f;
fj_i(i,:)=zeros(1,N);
fj_i(:,i)=zeros(N,1);
for j=[1:i-1 i+1:N]
W(j,:)=dc*s*(f(j,:)*P-P(j,:)*sum(f(j,:)))/(2*L); %coefficients of future prices
Z(j,:)=(fj_i(j,:)*P-P(j,:)*sum(f(j,:)))/(2*L); %coefficients of competitors prices.
%old version 
%Z(j,:)=-f(i,j)/(2*L)*P(j,:); %coefficients of competitors prices.
%Z(j,:)=Z(j,:)+(fj_i(j,:)*P-P(j,:)*sum(fj_i(j,:)))/(2*L);
end
W(i,:)=wi; %Matrix D_{k,x+1}^{young}
Z(i,:)=zi; %Matrix D_{k,x}^{young}
dyjp(i)=dyip; %d_{k,p}^{young}
dyj1(i)=dyi1;
Z=Z+repmat(dyj1,1,N); %add constant.

Zt=inv(eye(N)-W*(free^(-1)))*Z; %Z_{kx}*free
vt=inv(eye(N)-W*(free^(-1)))*dyjp; %z_{kx}*free
Yi=Zt(i,:);
Yip=vt(i,:);
Zt=Zt*(free^(-1)); %Z_{kx}
vt=vt*(free^(-1)); %z_{kx}
% Why did I normalize Zt and vt? Ok because Yi, and Yip enter demand and
% therefore should be in absolute terms (as they result from the matrix
% inversion lines). Intead, Zt and vt are going to enter the transition
% matrix and therefore should be normalized properly.
%Best response;
ri=g*Yip+Oip; %Eq 85->-d_{k,p}
Ri=g*Yi+Oi+(1+g)*fmat(i,i); %Eq 86->d_{k,x}+(1+g)*f(k,k)
%Value function iteration
V=zeros(N,N); 
tolV=1e-11;
dist=tolV+1;

new_version=true;
if new_version
    % Linear-quadratic problem
    % Instantaneous payoff: a0+(a1'x)p+(a2)p^2
    % Value Function: x_{+1}'Vx_{+1}
    % Transition Function: x_{+1}=Tx+rp
    %
    % Functional equation:
    % x'Vx=max{a0+(a1'x)p+(a2)p^2+df*x_{+1}'Vx_{+1}}
    %     =max{a0+(a1'x)p+(a2)p^2+df*(Tx+rp)'V(Tx+rp)}
    %     =max{a0+(a1'x)p+(a2)p^2+df*(r'Vrp^2+x'T'VTx+p2r'VTx)}
    %     =max{(a0+df x'T'VTx)+(a1'+2df r'VT)x p+(a2+df r'Vr)p^2}
    %     =max{(a0+df x'T'VTx)+(b1'        )x p+(b2        )p^2}
    % Solution: p=-b1'x/(2 b2)
    % (a0+df x'T'VTx)-x' b1b1'x /(2 b2)+ x'b1b1'x/(4 b2)
    % (a0+df x'T'VTx)- x'b1b1'x/(4 b2)
    % (a0+x' (df T'VT-b1b1'/(4b2)) x
    a0=-c(i)*Ri';
    a1=Ri'-c(i)*ri;
    a2=ri;
    T=Zt;
    r=vt; 
    while dist>tolV;
    [dist V Pti]=LQIteration(V);
    end
    if a2+df*r'*V*r>0; disp('2nd derivative=positive' ); end


else
    while dist>tolV;
    [dist V Pti]=iterV3(V);
    end
    if (2*(df*vt'*V*vt+ri))>0; disp('2nd derivative=positive' ); end
end

function [dist Vout Pti]=iterV3(Vin)
Vin=(Vin+Vin')/2-mean(mean(Vin)); %Symetrization and normalization
Pti=-((df*2*Zt'*Vin*vt+Ri')'-ri*c(i))/(2*(df*vt'*Vin*vt+ri)); %Eq 89
%disp([2*(df*vt'*Vin*vt+ri) absV])
Vti=Pti'*(df*vt'*Vin*vt+ri)*Pti+df*Zt'*Vin*Zt+(df*2*Zt'*Vin*vt+Ri')*Pti; %Construct new value function Eq.90
px=-c(i)*(Ri+ri*Pti);
Vti=Vti+(repmat(px',1,N)+repmat(px,N,1))/2;
Vti=(Vti+Vti')/2-mean(mean(Vti));
dist=max(max(abs(Vti-V)));
Vout=Vti;
end


function [dist Vout p]=LQIteration(V)
b1=(a1'+2*df*r'*V*T)';
b2=a2+df*r'*V*r;
Vout=(repmat(a0,1,N)+repmat(a0',N,1))/2+df*T'*V*T-b1*b1'/(4*b2);
dist=max(max(abs(Vout-V)));
p=-b1'/(2*b2);
end


end
