% findMPNEg Finds a MPNE in linear strategies for the General Assymetric Case
% [P ss Pdist mdist]=findMPNE(fmat,c,s,dc,df,gamma,firmtype,P)
% This code finds the MPNE for a given set of parameters:
% fmat: fmat has to be a NxN symmetric matrix and its lower (upper) triangular matrix
% has to add to one. Fij should be the density of the segment ij (N is the
% number of firms). Fii is the density of the segments within the firm i.
% c: vector of order N with the costs of each firm
% s: scalar. Switching cost (Length of segment=1).
% dc: scalar. Consumers' discount rate
% df: scalar. Firms' effective discount rate.
% g: scalar. Size of the young cohort
% firmtype: Nx1 vector of integers. Two firms with the same firmtype
% are supposed to be of the same type (The code should be able to construct this
% vector with the data provided above, but for the moment we need to
% provide it).
% P: N-by-N matrix. Initial guess for the matrix of linear strategies. P is
% such that Px is the vector of prices (when c=0).
% Output:
% P: N-by-N matrix. Fixed point for the matrix of linear strategies. P is
% such that Px is the vector of prices (when c=0). 
% ss: shares in the steady state of the equilibrium
% Pdist: the distance between the last two iterations of P (metric:
% max(max(A-B)))
% mdist: the max distance calculated for any bestresponse.
% Created by Paulo Somaini on December 3rd, 2009.
% Dependent Files: bestRespAsymg.m

function [P ss Pdist mdist]=findMPNEg(fmat,c,s,dc,df,g,firmtype,P)
L=1;
% Number of active firms;
N=size(fmat,1);
% Check fmat;
isSym=max(max(abs(fmat-fmat')));
sumOne=abs(sum(sum(tril(fmat)))-1);
if isSym+sumOne>1e-9; error('fmat has to be symmetric and its lower (upper) triangular matrix has to add to one'); end

% Firms;
firmtype=ceil(firmtype);
TolPd=1e-10;
Pdist=1+TolPd;
mdist=0;
while Pdist>TolPd;
Pold=P;
[b1, m1, n1]=unique(firmtype, 'first');
for type=1:length(b1);
    i=m1(type);
    [Pti dist]=bestRespAsymg(P,fmat,c,s,dc,df,g,i);
    % Assign values of the matrix P;
    P(i,:)=Pti;
    samet=find(type==n1);
    samet(1)=[];
    for firmt=samet;
        Ptib=Pti;
        Ptib(firmt)=Pti(i);
        Ptib(i)=Pti(firmt);
        P(firmt,:)=Ptib;
    end
mdist=max(mdist,dist);
end
Pdist=max(max(abs(P-Pold)));
end
%disp(P)

% Transition Matrix;
f=fmat-tril(triu(fmat));
free=sum(sum(f))/2;
Zd=zeros(N,N);
Wd=zeros(N,N);
for j=1:N;
Wd(j,:)=dc*s*(f(j,:)*P-P(j,:)*sum(f(j,:)))/(2*L);
Zd(j,:)=(f(j,:)*P-P(j,:)*sum(f(j,:)))/(2*L)+sum(f(j,:))/2;
end
trans=inv(eye(N)*free-Wd)*Zd;
[vector,value] = eig(trans);
eig1=(abs(diag(value)-1)<1e-9);
ss=(vector(:,eig1)); 
ss=ss/sum(ss);
end


